
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###** R code as supplemental material for Gonçalves et al. 2022
#
###** Amphibian roadkill estimation for EGR project
#
### Dail-Madsen open population model for unmarked individuals
###  with a dependent double observer protocol
###   to estimate road mortality at multiple sites
# 
##* Author: Ismael V. Brack Aug/2022
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#* 50 sites (road stretches)
#* double dependent observers in (up to) 6 visits

#* Data are available at the repository: 

library(jagsUI)
library(ggplot2)
devtools::source_url("https://github.com/ismaelvbrack/predictJAGS_func/blob/main/func_predictJAGS.R?raw=TRUE") # predictJAGS function

# Import data -------------------------------------------------------------

#* Count data
counts1 <- read.table("counts_camp1.txt",header=F)

#* Site covars
siteCovars <- read.table("siteCovs_2021.txt",header=T)
siteCovars$Trafego <- factor(siteCovars$Trafego, levels=c("baixo","medio","alto"))

#* Intervals covars
turnos1 <- read.table("turnos_camp1.txt",header=T)
rain1 <- read.table("pluv_camp1.txt",header=T)
intervals1 <- ifelse(turnos1[,2:(ncol(turnos1)-1)]=="tarde","night","day")


# Arranging data ----------------------------------------------------------

#* Arranging data
S <- nrow(counts1)   # number of sites
J <- ncol(counts1)/2 # number of visits

#* Count data
Y1 <- array(as.matrix(counts1),dim=c(S,2,J),
            dimnames=list(siteCovars$Trecho,c("obs1","obs2"),1:J))

# counts by both obs
ncap1 <- array(dim=c(S,J))
for (j in 1:J) {
  for (i in 1:nrow(Y1[,,j])) {
    ncap1[i,j] <- sum(Y1[i,,j])
  }} 

#* Gathering data
data1 <- list(Y=Y1,ncap=ncap1, # counts
              night=ifelse(intervals1=="night",1,0),day=ifelse(intervals1=="day",1,0), # visits covs
              rain=ifelse(rain1[,2:ncol(rain1)]=="chuva",1,0),
              urban=siteCovars$urban,wetland=siteCovars$wetland,traffic=as.numeric(siteCovars$Trafego), # site covs
              new.wet=new.wet, # data to predict entries for non-sampled sites
              S=S,J=J) # sampling design

#* Define initial values
# Initial values session 1
inity1 <- ncap1+3

inity1[which(is.na(inity1), arr.ind=T)] <- inity1[which(is.na(inity1),arr.ind=T)[,1],2]


N1.1 <- inity1
N1.1[,2:J] <- NA

Pers.in1 <- Entrs.in1 <- array(,c(S,J))
for (i in 1:S){
  for(j in 2:J){
    Pers.in1[i,j] <- rbinom(1,inity1[i,j-1],.5) # keep ~half individuals
    Entrs.in1[i,j] <-  ifelse((inity1[i,j]-Pers.in1[i,j])>0, (inity1[i,j]-Pers.in1[i,j]), 0) 
  }}

inits1 <- function(){list(alpha.lam=rnorm(1),betaUrb.lam=rnorm(1),betaWet.lam=rnorm(1), # lambda
                          alpha.gam=rnorm(1),betaWet.gam=rnorm(1),betaRain.gam=rnorm(1), # gamma
                          alpha.phi=rnorm(3),betaDay.phi=rnorm(1), # phi
                          p0=runif(1,0,1), # p
                          Persists=Pers.in1,Entries=Entrs.in1,N=N1.1,z=rep(1,S))}

#* Parameters monitored
params <- c("alpha.lam","betaUrb.lam","betaWet.lam",
            "alpha.gam","betaWet.gam","betaRain.gam",
            "alpha.phi","betaDay.phi",
            "p0",
            #"Entries",
            "siteFatals",#"new.siteFatals",
            "AllFatals")#,"new.AllFatas")


# Writing JAGS model ------------------------------------------------------

sink("DM-depObs_fullCovs.txt")
cat("
    model{
##* Priors -------------

# Initial abundance
alpha.lam ~ dnorm(0,0.1) # intercept for lambda
betaUrb.lam ~ dnorm(0,0.1) # effect of urban
betaWet.lam ~ dnorm(0,0.1) # effect of wetland

# Entries
alpha.gam ~ dnorm(0,0.1) # intercept for gamma
betaWet.gam ~ dnorm(0,0.1) # effect of wetland
betaRain.gam ~ dnorm(0,0.1) # effect of rain/no-rain (binary)

# Departures / Persistence
for(i in 1:3){
  alpha.phi[i] ~ dnorm(0,0.1) # intercepts for phi
}                              # one for each traffic category (low, medium, high)
betaDay.phi ~ dnorm(0,0.1) # effect of day/night

# Detection
p0 ~ dunif(0,1)

##* Likelihood -------------
for (i in 1:S){ #each site
  
  ### State variable process
  #* initial abundance j=1
  N[i,1] ~ dpois(lambda[i])
  
  log(lambda[i]) <- alpha.lam + betaUrb.lam*urban[i] + betaWet.lam*wetland[i]
  
  # dynamics j>1
    for(j in 2:J){
      # Persistence
      Persists[i,j] ~ dbin(phi[i,j],N[i,j-1])
      logit(phi[i,j]) <- alpha.phi[traffic[i]] + betaDay.phi*day[i,j-1]
      
      # Entries
      Entries[i,j] ~ dpois(gamma[i,j])
      log(gamma[i,j]) <- alpha.gam + betaWet.gam*wetland[i] + betaRain.gam*rain[i,j-1]
      
      # Abundance
      N[i,j] <- Persists[i,j] + Entries[i,j]
    }#j
    
   ## Conditional multinomial cell probabilities
   for(j in 1:J){
     p[i,j] <- p0 # detection prob.
     pi[i,1,j] <- p[i,j] # 1st observer
     pi[i,2,j] <- (1-p[i,j]) * p[i,j] # 2nd observer
     pi0[i,j] <- 1 -  pi[i,1,j] -  pi[i,2,j] # prob. of non-detection
     pcap[i,j] <- 1 - pi0[i,j] # prob. of at least one observer to detect
     
     pic[i,1,j] <- pi[i,1,j] / pcap[i,j] # conditional prob.1
     pic[i,2,j] <- pi[i,2,j] / pcap[i,j] # conditional prob.2
     
   ## Observation process
     ncap[i,j] ~ dbin(pcap[i,j] , N[i,j])
     Y[i,1:2,j] ~ dmulti(pic[i,1:2,j] , ncap[i,j])
     
  }#j
  
#* Derived -------------
  # All entries/fatalities per site
  siteFatals[i] <- sum(Entries[i,2:J])
  
}#i

  # All fatalities in sampled sites
  AllFatals <- sum(siteFatals[])
  
}",fill=TRUE)
sink()



# Running... --------------------------------------------------------------

#* MCMC settings 
na <- 10000
ni <- 100000
nt <- 1
nb <- 20000
nc <- 3

# Run!
out1 <- jags(data1, inits1, params, "DM-depObs_fullCovs.txt",
             n.chains=nc,n.thin=nt, n.iter=ni, n.burnin=nb, n.adapt=na, parallel=T)


# See results -------------------------------------------------------------
site.length <- 0.1 # in km
time.int <- 0.5 # in days

#* Parameter estimates
print(out1, digits=2)

#* Traceplots
traceplot(out1, param="AllFatals")
traceplot(out1, param="alpha.phi")

# Roadkills per day for each site
siteFatalsDay <- as.data.frame(t(apply(out1$sims.list$siteFatals / (time.int*(J-1)), 2,
                                       function(x) c(mean(x),quantile(x,probs=c(0.025,0.5,0.975))))))
names(siteFatalsDay) <- c("mean","lower","median","upper")

siteFatalsDay <- cbind(Trecho=siteCovars$Trecho,siteFatalsDay)

siteFatalsDay <- cbind(siteFatalsDay,
                      t(apply(ncap1,1,function(x) c(c.mean=mean(x,na.rm=T),c.max=max(x,na.rm=T)))))

# Roadkills per km per day for all sites (average)
c(mean=mean(out1$sims.list$AllFatals / (time.int*(J-1)) / (S*site.length)),
  quantile(out1$sims.list$AllFatals / (time.int*(J-1)) / (S*site.length),probs=c(0.025,0.5,0.975)))


# Relationship predictions ------------------------------------------------

# Initial abundance (lambda)
# ~ wetland

# wetland values to predict
new.wet <- cbind(urban=median(siteCovars$urban),
                 wetland=seq(min(siteCovars$wetland),max(siteCovars$wetland),,100)
)

pred.lamWet <- predictJAGS(fm=out1,params=c("alpha.lam","betaUrb.lam","betaWet.lam"),
                           newdata=new.wet,link="log",quants=c(0.025,0.5,0.975),appendData=T)

names(pred.lamWet)[2:4] <- c("lower","median","upper")

#* Figure! lambda ~ wetland
fig.lamWet <-  ggplot(pred.lamWet, aes(x=wetland,y=median,ymax=upper,ymin=lower)) +
  geom_line(size=1.4, color="darkgreen") + geom_ribbon(alpha=0.4, fill="darkgreen") +
  theme_classic() + #coord_cartesian(ylim=c(0,100)) +
  labs(x="% of wetlands (200m buffer)",y="Initial number of carcasses") +
  scale_x_continuous(breaks=seq(.1,.9,0.2),labels=seq(10,90,20)) +
  scale_y_continuous(limits=c(0,12),breaks=seq(0,12,2)) +
  annotate("text",label="a",x=0,y=12,size=7) +
  theme(axis.title=element_text(size=14),axis.text=element_text(size=12))

# Initial abundance (lambda)
# ~ urban

# urban values to predict
new.urb <- cbind(urban=seq(min(siteCovars$urban),max(siteCovars$urban),,100),
                 wetland=median(siteCovars$wetland))

# predict
pred.lamUrb <- predictJAGS(fm=out1,params=c("alpha.lam","betaUrb.lam","betaWet.lam"),
                           newdata=new.urb,link="log")

names(pred.lamUrb)[2:4] <- c("lower","median","upper")

#* Figure! lambda ~ urban
fig.lamUrb <-  ggplot(pred.lamUrb, aes(x=urban,y=median,ymax=upper,ymin=lower)) +
  geom_line(size=1.4, color="black") + geom_ribbon(alpha=0.4, fill="gray30") +
  theme_classic() + #coord_cartesian(ylim=c(0,100)) +
  labs(x="% of urban areas (200m buffer)",y="Initial number of carcasses") +
  scale_x_continuous(breaks=seq(0,.3,0.1),labels=seq(0,30,10)) +
  annotate("text",label="b",x=0,y=8,size=7) +
  theme(axis.title=element_text(size=14),axis.text=element_text(size=12))

# Entries (gamma)
# ~ wetland + rain

# wetland values to predict
new.wet2 <- data.frame(wetland=rep(seq(min(siteCovars$wetland),max(siteCovars$wetland),,100),2),
                       rain=rep(c(0,1),each=100))

pred.gamWet <- predictJAGS(fm=out1,params=c("alpha.gam","betaWet.gam","betaRain.gam"),
                           newdata=new.wet2,link="log",quants=c(0.025,0.5,0.975),appendData=T)

names(pred.gamWet)[2:4] <- c("lower","median","upper")

pred.gamWet$rain <- factor(pred.gamWet$rain,labels=c("no-rain","rain"))

#* Figure! lambda ~ wetland
fig.gamWet <-  ggplot(pred.gamWet, aes(x=wetland,y=median,ymax=upper,ymin=lower)) +
  geom_line(aes(color=rain),size=1.4) + geom_ribbon(aes(fill=rain),alpha=0.4) +
  theme_classic() + #ylim(0,16) +
  scale_color_manual(values=c("tan4","blue4")) +
  scale_fill_manual(values=c("tan3","blue")) +
  labs(x="% of wetlands (200m buffer)",y="Entrant carcasses / 12h") +
  scale_x_continuous(breaks=seq(.1,.9,0.2),labels=seq(10,90,20)) +
  scale_y_continuous(limits=c(0,16),breaks=seq(0,16,4)) +
  annotate("text",label="c",x=0,y=16,size=7) +
  theme(axis.title=element_text(size=14),axis.text=element_text(size=12),
        legend.title=element_blank(),legend.text=element_text(size=12),
        legend.position="top")

# Persistence (phi)
# ~ traffic + day

pred.phi <- out1$summary[grep("alpha.phi",rownames(out1$summary)),
                         c("mean","2.5%","50%","97.5%")]
pred.phi <- as.data.frame(plogis(rbind(pred.phi, 
                                       pred.phi + 
                                         rep(out1$summary["betaDay.phi",c("mean","2.5%","50%","97.5%")],each=nrow(pred.phi)))))
pred.phi$traffic <- rep(c("baixo","medio","alto"),2)
pred.phi$turno <- rep(c("night","day"),each=3)
names(pred.phi)[2:4] <- c("lower","median","upper")

pred.phi$traffic <- factor(pred.phi$traffic, levels=c("baixo","medio","alto"), labels=c("low","medium","high"))

fig.phi <- ggplot(pred.phi, aes(x=traffic, y=mean,ymin=lower,ymax=upper,color=turno))+
  geom_point(size=3,position=position_dodge(0.3)) + 
  geom_errorbar(width=0.3,size=1,position=position_dodge(0.3)) + 
  theme_classic() + scale_y_continuous(breaks=seq(0,1,0.2)) +
  labs(x="Traffic",y="Carcass persistence prob.",color="") +
  scale_color_manual(values=c("red3","blue3")) +
  annotate("text",label="d",x=0.6,y=1,size=7) +
  theme(axis.title=element_text(size=14),axis.text=element_text(size=12),
        legend.text=element_text(size=12),legend.position="top")
